clear all;
close all

%%
FSR=851.934; %MHz for green, https://dcc.ligo.org/DocDB/0140/T1700104/002/vopoFDD_v2.pdf
%FSR=853.452; %MHz for IR, https://dcc.ligo.org/DocDB/0140/T1700104/002/vopoFDD_v2.pdf

%% load data
tmp=load('TEK00000.CSV'); %PD output for OPO green transmission
tstamp=tmp(:,1);
trans=tmp(:,2);
tmp=load('TEK00002.CSV'); %HV driver input
HV=tmp(:,2)*15; % 15 is the HV amplifier gain of Thorlabs MDT694

%% find all 00 peaks in the transmission.
transcopy=trans;

pthresh=0.3; %00 peaks need to be larger than 0.3V.
pwidth=(-10:10)'; %00 peaks are narrower than +-10 samples.
peaks=zeros(size(trans));
peakfound=true;
while peakfound
    [m, i]=max(transcopy);
    if m>pthresh
        peaks(i)=1;
        transcopy(i+pwidth)=zeros(size(pwidth));
    else
        peakfound=false;
    end
end
% peaks==1 when there is a 00 mode peak, otherwise 0.
peaksIdx=peaks==1;

figure(1)
subplot(211)
plot(tstamp, trans, 'b-', tstamp(peaksIdx), 0, 'ro')
ylabel('V')
title('OPO transmission (green)')
subplot(212)
plot(tstamp, HV);
title('PZT4 voltage')
ylabel('V')
xlabel('second')

%% identify main HV slopes in the data.
HVcopy=zeros([length(HV)+2000, 1]); %adding 1000 samples of zero in front and at the end.
HVcopy(1001:(1000+length(HV)))=HV;
% voltage went up, down, up and down. We want to select the longest slopes,
% one declining and the other increasing.
[m,maxIdx1]=max(HVcopy);
HVcopy(maxIdx1+(-1000:1000))=zeros(size(-1000:1000));
[m,maxIdx2]=max(HVcopy);
if maxIdx1>maxIdx2
    tmp=maxIdx1;
    maxIdx1=maxIdx2;
    maxIdx2=tmp;
end
maxIdx1=maxIdx1-1000;
maxIdx2=maxIdx2-1000;
%
[m, minIdx]=min(HV);

% down slope we're interested in is HV(i) 
%   where maxIdx1<=i<minIdx.
% up slope we're interested in hs HV(i) 
%   where minIdx<=i<maxIdx2.

%% First look at the down slope.
mask=(1:length(HV))>=maxIdx1 & (1:length(HV))<minIdx;
ts_DS=tstamp(mask);
downslope=HV(mask);
idx=1:length(downslope);
p=polyfit(ts_DS, downslope, 1);
fitval_DS=p(2)+p(1)*ts_DS;

peak_DS=peaks(mask);
peakmask_DS=peak_DS==1;
HVAtPeaks_DS=fitval_DS(peakmask_DS);
freq_DS=zeros(size(HVAtPeaks_DS));
for ii=2:length(freq_DS)
    freq_DS(ii)=freq_DS(ii-1)-FSR;
end
%freq_DS=0 just means that the frequency corresponds to the first resonance
%in the down slope.

%what's the MHz/voltage?
p=polyfit(HVAtPeaks_DS, freq_DS, 1);
PZTCal_DS=p(1);

figure(1)
subplot(212)
hold on
plot(ts_DS, fitval_DS, 'r')

%% Look at the up slope.
mask=(1:length(HV))>=minIdx & (1:length(HV))<maxIdx2;
ts_US=tstamp(mask);
upslope=HV(mask);
idx=1:length(upslope);
p=polyfit(ts_US, upslope, 1);
fitval_US=p(2)+p(1)*ts_US;

peak_US=peaks(mask);
peakmask_US=peak_US==1;
HVAtPeaks_US=fitval_US(peakmask_US);
freq_US=zeros(size(HVAtPeaks_US));
freq_US(1)=freq_DS(end);
for ii=2:length(freq_US)
    freq_US(ii)=freq_US(ii-1)+FSR;
end

%what's the MHz/voltage?
p=polyfit(HVAtPeaks_US, freq_US, 1);
PZTCal_US=p(1);

figure(1)
subplot( 212)
hold on
plot(ts_US, fitval_US, 'g')

%% 
figure(2)
plot(HVAtPeaks_DS, freq_DS, 'ro', HVAtPeaks_US, freq_US, 'bx')
xlabel('HV [V]')
ylabel('freq relative to one 00 peak [MHz]')
legend(['Data corresponding to down slope, ', num2str(PZTCal_DS), 'MHz/V'], ...
    ['Data corresponding to up slope, ', num2str(PZTCal_US), 'MHz/V']);
title(['PZT calibration, ', num2str(mean([PZTCal_DS, PZTCal_US])), 'MHz/V'])